import argparse
import os

import requests

JSON_DB_FILE = 'https://raw.githubusercontent.com/github/gemoji/master/db/emoji.json'

def generate(path, dbname):
    req = requests.get(JSON_DB_FILE)
    req.raise_for_status()

    data = req.json()

    path = os.path.join(path, dbname)

    with open(path, 'w') as file:
        file.write('### This is a generated file.\n')
        file.write('### Do not edit this file.\n')
        file.write('### This file is based on "{0}".\n'.format(JSON_DB_FILE))
        file.write('\n')
        file.write('from collections import namedtuple\n')
        file.write('\n')
        file.write('Emoji = namedtuple("Emoji", ["aliases", "emoji", "tags", "category"])\n')
        file.write('\n')
        file.write('EMOJI_DB = [\n')

        for emoji in data:
            if 'emoji' in emoji:
                file.write('    Emoji({aliases}, "{emoji}", {tags}, "{category}"),\n'.format(**{
                    'aliases': emoji['aliases'],
                    'emoji': emoji['emoji'],
                    'tags': emoji['tags'],
                    'category': emoji['category'],
                }))

        file.write(']\n')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Generates the Emoji database.')
    parser.add_argument('--dir', default='.', help='Database location')
    parser.add_argument('--dbname', default='db.py', help='Database location')
    args = parser.parse_args()

    generate(args.dir, args.dbname)



from . import db

def get_emoji_aliases():
    emoji_aliases = {}

    for emoji in db.EMOJI_DB:
        for alias in emoji.aliases:
            alias = ':{0}:'.format(alias)
            emoji_aliases[alias] = emoji.emoji

    return emoji_aliases


def get_emoji_by_code(code):
    try:
        return next(filter(lambda emoji: code == emoji.emoji, db.EMOJI_DB))
    except StopIteration:
        return None


def get_emoji_by_alias(alias):
    try:
        return next(filter(lambda emoji: alias in emoji.aliases, db.EMOJI_DB))
    except StopIteration:
        return None


def get_emojis_by_tag(tag):
    return filter(lambda emoji: tag in emoji.tags, db.EMOJI_DB)


def get_emojis_by_category(category):
    return filter(lambda emoji: category.lower() == emoji.category.lower(), db.EMOJI_DB)


def get_tags():
    return sorted({tag for emoji in db.EMOJI_DB for tag in emoji.tags})

def get_categories():
    return sorted({emoji.category for emoji in db.EMOJI_DB})



import re

from .db import utils

ALIAS_TO_EMOJI = utils.get_emoji_aliases()
EMOJI_TO_ALIAS = dict((v, k) for k, v in ALIAS_TO_EMOJI.items())

RE_TEXT_TO_EMOJI_GROUP = '({0})'.format('|'.join(ALIAS_TO_EMOJI))
RE_TEXT_TO_EMOJI = re.compile(RE_TEXT_TO_EMOJI_GROUP)

RE_EMOJI_TO_TEXT_GROUP = '({0})'.format('|'.join([re.escape(emoji) for emoji in EMOJI_TO_ALIAS]))
RE_EMOJI_TO_TEXT = re.compile(RE_EMOJI_TO_TEXT_GROUP)


def encode(msg):
    msg = RE_TEXT_TO_EMOJI.sub(lambda match: ALIAS_TO_EMOJI[match.group(0)], msg)
    return msg


def decode(msg):
    msg = RE_EMOJI_TO_TEXT.sub(lambda match: EMOJI_TO_ALIAS[match.group(0)], msg)
    return msg


def get(msg):
    return {match.group() for match in  RE_EMOJI_TO_TEXT.finditer(msg)}


def count(msg, unique=False):
    if unique:
        return len({match.group() for match in  RE_EMOJI_TO_TEXT.finditer(msg)})
    return len([match.group() for match in  RE_EMOJI_TO_TEXT.finditer(msg)])
